Index: src/ia32/ic-ia32.cc |
diff --git a/src/ia32/ic-ia32.cc b/src/ia32/ic-ia32.cc |
index a83db129a1db4532a7825cd9e051759bf5a00b8c..82bb02a318c02d87352ef15516f189bb74a7f8fc 100644 |
--- a/src/ia32/ic-ia32.cc |
+++ b/src/ia32/ic-ia32.cc |
@@ -1374,10 +1374,10 @@ void StoreIC::GenerateArrayLength(MacroAssembler* masm) { |
// -- esp[0] : return address |
// ----------------------------------- |
// |
// This accepts as a receiver anything JSObject::SetElementsLength accepts |
// (currently anything except for external arrays which means anything with |
- // elements of FixedArray type.), but currently is restricted to JSArray. |
- // Value must be a number, but only smis are accepted as the most common case. |
+ // elements of FixedArray type). Value must be a number, but only smis are |
+ // accepted as the most common case. |
Label miss; |
@@ -1399,6 +1399,13 @@ void StoreIC::GenerateArrayLength(MacroAssembler* masm) { |
__ CmpObjectType(scratch, FIXED_ARRAY_TYPE, scratch); |
__ j(not_equal, &miss); |
+ // Check that the array has fast properties, otherwise the length |
+ // property might have been redefined. |
+ __ mov(scratch, FieldOperand(receiver, JSArray::kPropertiesOffset)); |
+ __ CompareRoot(FieldOperand(scratch, FixedArray::kMapOffset), |
+ Heap::kHashTableMapRootIndex); |
+ __ j(equal, &miss); |
+ |
// Check that value is a smi. |
__ JumpIfNotSmi(value, &miss); |
@@ -1374,10 +1374,10 @@ void StoreIC::GenerateArrayLength(MacroAssembler* masm) { |
// ----------------------------------- |
// |
- // This accepts as a receiver anything JSObject::SetElementsLength accepts |
+ // This accepts as a receiver anything JSArray::SetElementsLength accepts |
// (currently anything except for external arrays which means anything with |